@@ -14,8 +14,6 @@ Contains the motion scanning engine (`MotionScanner`) for DVR-Scan.
 """
 
 import logging
-import os
-import os.path
 import queue
 import subprocess
 import sys
@@ -23,7 +21,7 @@ import threading
 import typing as ty
 from dataclasses import dataclass
 from enum import Enum
-from typing import Any, AnyStr, List, Optional, Tuple, Union
+from pathlib import Path
 
 import cv2
 import numpy as np
@@ -36,7 +34,6 @@ from dvr_scan.overlays import BoundingBoxOverlay, TextOverlay
 from dvr_scan.platform import (
     HAS_PILLOW,
     HAS_TKINTER,
-    get_filename,
     get_min_screen_bounds,
     is_ffmpeg_available,
 )
@@ -61,13 +58,15 @@ MAX_ENCODE_QUEUE_SIZE: int = 4
 DEFAULT_FFMPEG_INPUT_ARGS = "-v error"
 """Default arguments to add before input when invoking ffmpeg."""
 
-DEFAULT_FFMPEG_OUTPUT_ARGS = "-map 0 -c:v libx264 -preset fast -crf 21 -c:a aac -sn"
+DEFAULT_FFMPEG_OUTPUT_ARGS = (
+    "-map 0:v:0 -map 0:a? -map 0:s? -c:v libx264 -preset veryfast -crf 22 -c:a aac"
+)
 """Default arguments passed to ffmpeg when using OutputMode.FFMPEG."""
 
-COPY_MODE_OUTPUT_ARGS = "-map 0 -c:v copy -c:a copy -sn"
+COPY_MODE_OUTPUT_ARGS = "-map 0:v:0 -map 0:a? -map 0:s? -c:v copy -c:a copy"
 """Default arguments passed to ffmpeg when using OutputMode.COPY."""
 
-# TODO(1.8): Add ability to set output name template."
+# TODO(#89): Add ability to set output name template.
 OUTPUT_FILE_TEMPLATE = "{VIDEO_NAME}.DSME_{EVENT_NUMBER}.{EXTENSION}"
 """Template to use for generating output files."""
 
@@ -99,11 +98,11 @@ class OutputMode(Enum):
         if self == OutputMode.SCAN_ONLY:
             return "SCAN_ONLY"
         if self == OutputMode.OPENCV:
-            return "SCAN_ONLY"
+            return "OPENCV"
         if self == OutputMode.COPY:
-            return "SCAN_ONLY"
+            return "COPY"
         if self == OutputMode.FFMPEG:
-            return "SCAN_ONLY"
+            return "FFMPEG"
 
 
 @dataclass
@@ -121,7 +120,7 @@ class EncodeFrameEvent:
     frame_bgr: np.ndarray
     timecode: FrameTimecode
     score: float
-    bounding_box: Tuple[int, int, int, int]
+    bounding_box: ty.Tuple[int, int, int, int]
     end_of_event: bool = False
 
 
@@ -132,7 +131,7 @@ class MotionMaskEvent:
     motion_mask: np.ndarray
     timecode: FrameTimecode
     score: float
-    bounding_box: Optional[Tuple[int, int, int, int]]
+    bounding_box: ty.Optional[ty.Tuple[int, int, int, int]]
 
 
 @dataclass
@@ -147,7 +146,7 @@ class MotionEvent:
 class DetectionResult:
     """Motion events detected from scanning `num_frames` consecutive frames."""
 
-    event_list: List[MotionEvent]
+    event_list: ty.List[MotionEvent]
     num_frames: int
 
 
@@ -167,15 +166,15 @@ def _recommended_kernel_size(frame_width: int, downscale_factor: int) -> int:
 
 
 def _extract_event_ffmpeg(
-    input_path: AnyStr,
-    output_path: AnyStr,
+    input_path: Path,
+    output_path: Path,
     start_time: FrameTimecode,
     end_time: FrameTimecode,
     ffmpeg_input_args: str,
     ffmpeg_out_args: str,
     log_args: bool = False,
 ) -> bool:
-    args: List[str] = [
+    args: ty.List[str] = [
         "ffmpeg",
         "-y",
         "-nostdin",
@@ -183,16 +182,13 @@ def _extract_event_ffmpeg(
         "-ss",
         start_time.get_timecode(),
         "-i",
-        input_path,
+        str(input_path),
         "-t",
         (end_time - start_time).get_timecode(),
         *ffmpeg_out_args.split(" "),
-        output_path,
+        str(output_path),
     ]
-    # Log the arguments we're passing to ffmpeg.
-    if logger.getEffectiveLevel() == logging.DEBUG:
-        logger.debug("Running ffmpeg, args:\n  %s", " ".join(args))
-    elif log_args:
+    if log_args or logger.getEffectiveLevel() == logging.DEBUG:
         logger.info("%s", " ".join(args))
     # Invoke the command and capture the output (exception is raised on non-zero return code).
     output: str = subprocess.check_output(
@@ -222,7 +218,7 @@ class MotionScanner:
 
     def __init__(
         self,
-        input_videos: List[AnyStr],
+        input_videos: ty.List[Path],
         input_mode: str = "opencv",
         frame_skip: int = 0,
         show_progress: bool = False,
@@ -234,13 +230,13 @@ class MotionScanner:
         # Scan state and options they come from:
 
         # Output Parameters (set_output)
-        self._comp_file: Optional[AnyStr] = None  # -o/--output
-        self._mask_file: Optional[AnyStr] = None  # -mo/--mask-output
-        self._fourcc: Any = None  # opencv-codec
+        self._comp_file: ty.Optional[Path] = None  # -o/--output
+        self._mask_file: ty.Optional[Path] = None  # -mo/--mask-output
+        self._fourcc: ty.Any = None  # opencv-codec
         self._output_mode: OutputMode = None  # -m/--output-mode / -so/--scan-only
-        self._ffmpeg_input_args: Optional[str] = None  # input args for OutputMode.FFMPEG/COPY
-        self._ffmpeg_output_args: Optional[str] = None  # output args for OutputMode.FFMPEG
-        self._output_dir: AnyStr = ""  # -d/--directory
+        self._ffmpeg_input_args: ty.Optional[str] = None  # input args for OutputMode.FFMPEG/COPY
+        self._ffmpeg_output_args: ty.Optional[str] = None  # output args for OutputMode.FFMPEG
+        self._output_dir: ty.Optional[Path] = None  # -d/--directory
         # TODO: Replace uses of self._output_dir with
         # a helper function called "get_output_path".
 
@@ -257,6 +253,9 @@ class MotionScanner:
         self._downscale_factor = 1  # -df/--downscale-factor
         self._learning_rate = -1  # learning-rate
         self._max_threshold = 255.0  # max-threshold
+        self._max_area = 1.0  # max-area
+        self._max_width = 1.0  # max-width
+        self._max_height = 1.0  # max-height
 
         # Motion Event Parameters (set_event_params)
         self._min_event_len = None  # -l/--min-event-length
@@ -266,9 +265,9 @@ class MotionScanner:
 
         # Region Parameters (set_region)
         self._region_editor = False  # -w/--region-window
-        self._regions: List[List[Point]] = []  # -a/--add-region, -w/--region-window
-        self._load_region: Optional[str] = None  # -R/--load-region
-        self._save_region: Optional[str] = None  # -s/--save-region
+        self._regions: ty.List[ty.List[Point]] = []  # -a/--add-region, -w/--region-window
+        self._load_region: ty.Optional[Path] = None  # -R/--load-region
+        self._save_region: ty.Optional[Path] = None  # -s/--save-region
         self._max_roi_size_deprecated = None
         self._show_roi_window_deprecated = False
         self._roi_deprecated = None
@@ -283,9 +282,9 @@ class MotionScanner:
         self._stop: threading.Event = threading.Event()
         self._decode_thread_exception = None
         self._encode_thread_exception = None
-        self._video_writer: Optional[cv2.VideoWriter] = None
-        self._mask_size: Tuple[int, int] = None
-        self._mask_writer: Optional[cv2.VideoWriter] = None
+        self._video_writer: ty.Optional[cv2.VideoWriter] = None
+        self._mask_size: ty.Tuple[int, int] = None
+        self._mask_writer: ty.Optional[cv2.VideoWriter] = None
         self._num_events: int = 0
 
         # Thumbnail production (set_thumbnail_params)
@@ -303,6 +302,10 @@ class MotionScanner:
         self.set_thumbnail_params()
         self.set_video_time()
 
+    @property
+    def output_mode(self) -> OutputMode:
+        return self._output_mode
+
     @property
     def framerate(self) -> float:
         """Get the framerate of the currently loaded video(s)."""
@@ -310,10 +313,10 @@ class MotionScanner:
 
     def set_output(
         self,
-        output_dir: AnyStr = "",
-        comp_file: Optional[AnyStr] = None,
-        mask_file: Optional[AnyStr] = None,
-        output_mode: Union[OutputMode, str] = OutputMode.SCAN_ONLY,
+        output_dir: ty.Optional[Path] = None,
+        comp_file: ty.Optional[Path] = None,
+        mask_file: ty.Optional[Path] = None,
+        output_mode: ty.Union[OutputMode, str] = OutputMode.SCAN_ONLY,
         opencv_fourcc: str = DEFAULT_VIDEOWRITER_CODEC,
         ffmpeg_input_args: str = DEFAULT_FFMPEG_INPUT_ARGS,
         ffmpeg_output_args: str = DEFAULT_FFMPEG_OUTPUT_ARGS,
@@ -333,7 +336,7 @@ class MotionScanner:
                 XVID, MP4V, MP42, H264.
             ffmpeg_input_args: Arguments to pass to ffmpeg before the input video. Only used
                 when output_mode is OutputMode.FFMPEG or OutputMode.COPY.
-            ffmpeg_out_args: Arguments to pass to ffmpeg for the output video. Only used when
+            ffmpeg_output_args: Arguments to pass to ffmpeg for the output video. Only used when
                 output_mode is OutputMode.FFMPEG.
 
         Raises:
@@ -347,9 +350,9 @@ class MotionScanner:
              - output_mode does not exist in OutputMode
         """
         if output_dir:
-            if comp_file and os.path.isabs(comp_file):
+            if comp_file and comp_file.is_absolute():
                 raise ValueError("output file cannot be absolute path if output directory is set!")
-            if mask_file and os.path.isabs(mask_file):
+            if mask_file and mask_file.is_absolute():
                 raise ValueError("mask file cannot be absolute path if output directory is set!")
         if len(opencv_fourcc) != 4:
             raise ValueError("codec must be exactly FOUR (4) characters")
@@ -374,14 +377,14 @@ class MotionScanner:
         self._ffmpeg_output_args = ffmpeg_output_args
         # If an output directory is defined, ensure it exists, and if not, try to create it.
         if output_dir:
-            os.makedirs(output_dir, exist_ok=True)
+            output_dir.mkdir(parents=True, exist_ok=True)
         self._output_dir = output_dir
 
     def set_overlays(
         self,
-        timecode_overlay: Optional[TextOverlay] = None,
-        metrics_overlay: Optional[TextOverlay] = None,
-        bounding_box: Optional[BoundingBoxOverlay] = None,
+        timecode_overlay: ty.Optional[TextOverlay] = None,
+        metrics_overlay: ty.Optional[TextOverlay] = None,
+        bounding_box: ty.Optional[BoundingBoxOverlay] = None,
     ):
         """Sets options to use if/when drawing overlays on the resulting frames.
 
@@ -400,6 +403,9 @@ class MotionScanner:
         detector_type: DetectorType = DetectorType.MOG2,
         threshold: float = 0.15,
         max_threshold: float = 255.0,
+        max_area: float = 1.0,
+        max_width: float = 1.0,
+        max_height: float = 1.0,
         variance_threshold: float = 16.0,
         kernel_size: int = -1,
         downscale_factor: int = 1,
@@ -408,6 +414,9 @@ class MotionScanner:
         """Set detection parameters."""
         self._threshold = threshold
         self._max_threshold = max_threshold
+        self._max_area = max_area
+        self._max_width = max_width
+        self._max_height = max_height
         self._subtractor_type = detector_type
         if downscale_factor < 0:
             raise ValueError("Downscale factor must be positive.")
@@ -427,10 +436,10 @@ class MotionScanner:
     def set_regions(
         self,
         region_editor: bool = False,
-        regions: Optional[List[List[Point]]] = None,
-        load_region: Optional[str] = None,
-        save_region: Optional[str] = None,
-        roi_deprecated: Optional[List[int]] = None,
+        regions: ty.Optional[ty.List[ty.List[Point]]] = None,
+        load_region: ty.Optional[Path] = None,
+        save_region: ty.Optional[Path] = None,
+        roi_deprecated: ty.Optional[ty.List[int]] = None,
     ):
         """Set options for limiting detection regions."""
         self._region_editor = region_editor
@@ -460,9 +469,9 @@ class MotionScanner:
 
     def set_event_params(
         self,
-        min_event_len: Union[int, float, str] = "0.1s",
-        time_pre_event: Union[int, float, str] = "1.5s",
-        time_post_event: Union[int, float, str] = "2s",
+        min_event_len: ty.Union[int, float, str] = "0.1s",
+        time_pre_event: ty.Union[int, float, str] = "1.5s",
+        time_post_event: ty.Union[int, float, str] = "2s",
         use_pts: bool = False,
     ):
         """Set motion event parameters."""
@@ -477,9 +486,9 @@ class MotionScanner:
 
     def set_video_time(
         self,
-        start_time: Optional[Union[int, float, str]] = None,
-        end_time: Optional[Union[int, float, str]] = None,
-        duration: Optional[Union[int, float, str]] = None,
+        start_time: ty.Optional[ty.Union[int, float, str]] = None,
+        end_time: ty.Optional[ty.Union[int, float, str]] = None,
+        duration: ty.Optional[ty.Union[int, float, str]] = None,
     ):
         """Set a sub-set of the video in time for processing."""
         assert self._input.framerate is not None
@@ -504,7 +513,7 @@ class MotionScanner:
             return
 
         if self._load_region:
-            if not os.path.exists(self._load_region):
+            if not self._load_region.exists():
                 logger.error(f"File does not exist: {self._load_region}")
                 raise ValueError(
                     "Could not find specified region file. Ensure the specified path is valid "
@@ -519,7 +528,7 @@ class MotionScanner:
                     reason = "Could not parse region file!"
                 logger.error(f"Error loading region from {self._load_region}: {reason}")
             else:
-                logger.debug(
+                logger.info(
                     "Loaded %d region%s:\n%s",
                     len(region_editor),
                     "s" if len(region_editor) > 1 else "",
@@ -546,8 +555,7 @@ class MotionScanner:
                 raise SystemExit(1)
 
             logger.info("Selecting area of interest:")
-            # TODO(1.8): Ensure ROI window respects start time if set.
-            # TODO(1.8): We should process this frame (right now it gets skipped).
+            # TODO(#239): We should process this frame (right now it gets skipped).
             frame_for_crop = self._input.read()
             scale_factor = 1
             screen_bounds = get_min_screen_bounds()
@@ -586,7 +594,7 @@ class MotionScanner:
             )
             path = self._save_region
             if self._output_dir:
-                path = os.path.join(self._output_dir, path)
+                path = self._output_dir / path
             with open(path, "w") as region_file:
                 for shape in self._regions:
                     region_file.write(" ".join(f"{x} {y}" for x, y in shape))
@@ -635,12 +643,12 @@ class MotionScanner:
         self._scan_started = scan_started
         self._processed_frame = processed_frame
 
-    def scan(self) -> Optional[DetectionResult]:
+    def scan(self) -> ty.Optional[DetectionResult]:
         """Performs motion analysis on the MotionScanner's input video(s)."""
         self._stop.clear()
-        buffered_frames: List[np.ndarray] = []
-        event_window: List[float] = []
-        event_list: List[MotionEvent] = []
+        buffered_frames: ty.List[np.ndarray] = []
+        event_window: ty.List[float] = []
+        event_list: ty.List[MotionEvent] = []
         num_frames_post_event = 0
         event_start = None
 
@@ -670,6 +678,10 @@ class MotionScanner:
         else:
             kernel_size = _scale_kernel_size(self._kernel_size, self._downscale_factor)
 
+        test_width_height_area = (
+            self._max_area < 1.0 or self._max_width < 1.0 or self._max_height < 1.0
+        )
+
         # Create background subtractor and motion detector.
         # TODO: Figure out how to avoid logging unused parameters or emit a warning. For example,
         # the `variance_threshold` parameter is ignored by the `CNT` subtractor.
@@ -776,7 +788,7 @@ class MotionScanner:
                     num_events += 1
                 self._processed_frame(progress_bar=progress_bar, num_events=num_events)
             # Keep polling decode queue until it's empty (signaled via None).
-            frame: Optional[DecodeEvent] = decode_queue.get()
+            frame: ty.Optional[DecodeEvent] = decode_queue.get()
             if frame is None:
                 break
             assert frame.frame_bgr is not None
@@ -791,11 +803,25 @@ class MotionScanner:
                 )
             result = detector.update(frame.frame_bgr)
             frame_score = result.score
+
+            if test_width_height_area:
+                box_width = cv2.boundingRect(result.subtracted)[2] * self._downscale_factor
+                box_height = cv2.boundingRect(result.subtracted)[3] * self._downscale_factor
+                width_fraction = box_width / self._input.resolution[0]
+                height_fraction = box_height / self._input.resolution[1]
+                area_fraction = width_fraction * height_fraction
+
             # TODO: The rejection filter can be disabled by providing values > 255.0, but we should
             # provide a better method of disabling it. It might also be useful to allow users to
             # specify the amount of consecutive frames the filter can be active for.
             if frame_score >= self._max_threshold:
                 frame_score = 0
+            if test_width_height_area and (
+                area_fraction > self._max_area
+                or width_fraction > self._max_width
+                or height_fraction > self._max_height
+            ):
+                frame_score = 0
             above_threshold = frame_score >= self._threshold
 
             if above_threshold and frame_score > self._highscore:
@@ -854,21 +880,21 @@ class MotionScanner:
                             "event %d high score %f" % (1 + self._num_events, self._highscore)
                         )
                         if self._thumbnails == "highscore":
-                            video_name = get_filename(
-                                path=self._input.paths[0], include_extension=False
-                            )
-                            output_path = (
+                            video_name = self._input.paths[0].stem
+                            output_path: Path = (
                                 self._comp_file
                                 if self._comp_file
-                                else OUTPUT_FILE_TEMPLATE.format(
-                                    VIDEO_NAME=video_name,
-                                    EVENT_NUMBER="%04d" % (1 + self._num_events),
-                                    EXTENSION="jpg",
+                                else Path(
+                                    OUTPUT_FILE_TEMPLATE.format(
+                                        VIDEO_NAME=video_name,
+                                        EVENT_NUMBER="%04d" % (1 + self._num_events),
+                                        EXTENSION="jpg",
+                                    )
                                 )
                             )
                             if self._output_dir:
-                                output_path = os.path.join(self._output_dir, output_path)
-                            cv2.imwrite(output_path, self._highframe)
+                                output_path = self._output_dir / output_path
+                            cv2.imwrite(str(output_path), self._highframe)
                             self._highscore = 0
                             self._highframe = None
 
@@ -981,19 +1007,21 @@ class MotionScanner:
 
             logger.debug("event %d high score %f" % (1 + self._num_events, self._highscore))
             if self._thumbnails == "highscore":
-                video_name = get_filename(path=self._input.paths[0], include_extension=False)
+                video_name = self._input.paths[0].stem
                 output_path = (
                     self._comp_file
                     if self._comp_file
-                    else OUTPUT_FILE_TEMPLATE.format(
-                        VIDEO_NAME=video_name,
-                        EVENT_NUMBER="%04d" % (1 + self._num_events),
-                        EXTENSION="jpg",
+                    else Path(
+                        OUTPUT_FILE_TEMPLATE.format(
+                            VIDEO_NAME=video_name,
+                            EVENT_NUMBER="%04d" % (1 + self._num_events),
+                            EXTENSION="jpg",
+                        )
                     )
                 )
                 if self._output_dir:
-                    output_path = os.path.join(self._output_dir, output_path)
-                cv2.imwrite(output_path, self._highframe)
+                    output_path = self._output_dir / output_path
+                cv2.imwrite(str(output_path), self._highframe)
                 self._highscore = 0
                 self._highframe = None
 
@@ -1059,16 +1087,16 @@ class MotionScanner:
             # Make sure main thread stops processing loop.
             decode_queue.put(None)
 
-    def _init_video_writer(self, path: AnyStr, frame_size: Tuple[int, int]) -> cv2.VideoWriter:
+    def _init_video_writer(self, path: Path, frame_size: ty.Tuple[int, int]) -> cv2.VideoWriter:
         """Create a new cv2.VideoWriter using the correct framerate."""
         if self._output_dir:
-            path = os.path.join(self._output_dir, path)
+            path = self._output_dir / path
         effective_framerate = (
             self._input.framerate
             if self._frame_skip < 1
             else self._input.framerate / (1 + self._frame_skip)
         )
-        return cv2.VideoWriter(path, self._fourcc, effective_framerate, frame_size)
+        return cv2.VideoWriter(str(path), self._fourcc, effective_framerate, frame_size)
 
     def _on_encode_frame_event(self, event: EncodeFrameEvent):
         size = (event.frame_bgr.shape[1], event.frame_bgr.shape[0])
@@ -1082,14 +1110,16 @@ class MotionScanner:
             return
         if self._video_writer is None:
             # Use the first input video name as a filename template.
-            video_name = get_filename(path=self._input.paths[0], include_extension=False)
+            video_name = self._input.paths[0].stem
             output_path = (
                 self._comp_file
                 if self._comp_file
-                else OUTPUT_FILE_TEMPLATE.format(
-                    VIDEO_NAME=video_name,
-                    EVENT_NUMBER="%04d" % (1 + self._num_events),
-                    EXTENSION="avi",
+                else Path(
+                    OUTPUT_FILE_TEMPLATE.format(
+                        VIDEO_NAME=video_name,
+                        EVENT_NUMBER="%04d" % (1 + self._num_events),
+                        EXTENSION="avi",
+                    )
                 )
             )
             self._video_writer = self._init_video_writer(output_path, size)
@@ -1104,7 +1134,7 @@ class MotionScanner:
         frame: np.ndarray,
         timecode: FrameTimecode,
         frame_score: float,
-        bounding_box: Optional[Tuple[int, int, int, int]],
+        bounding_box: ty.Optional[ty.Tuple[int, int, int, int]],
         use_shift=True,
     ):
         if self._timecode_overlay is not None:
@@ -1156,18 +1186,20 @@ class MotionScanner:
             else COPY_MODE_OUTPUT_ARGS
         )
         # Use the first input video name as a filename template.
-        video_name = get_filename(path=self._input.paths[0], include_extension=False)
+        video_name = self._input.paths[0].stem
         output_path = (
             self._comp_file
             if self._comp_file
-            else OUTPUT_FILE_TEMPLATE.format(
-                VIDEO_NAME=video_name,
-                EVENT_NUMBER="%04d" % self._num_events,
-                EXTENSION="mp4",
+            else Path(
+                OUTPUT_FILE_TEMPLATE.format(
+                    VIDEO_NAME=video_name,
+                    EVENT_NUMBER="%04d" % self._num_events,
+                    EXTENSION="mp4",
+                )
             )
         )
         if self._output_dir:
-            output_path = os.path.join(self._output_dir, output_path)
+            output_path = self._output_dir / output_path
         # Only log the args passed to ffmpeg on the first event, to reduce log spam.
         log_args = False
         if self._num_events == 1:
@@ -1186,7 +1218,7 @@ class MotionScanner:
     def _encode_thread(self, encode_queue: queue.Queue):
         try:
             while True:
-                event: Optional[Union[EncodeFrameEvent, MotionMaskEvent, MotionEvent]] = (
+                event: ty.Optional[ty.Union[EncodeFrameEvent, MotionMaskEvent, MotionEvent]] = (
                     encode_queue.get()
                 )
                 if event is None:
